#!/usr/bin/perl
#
# Script to patch ABC80 rom with checksum 9913 to support 80 column text
#

use bytes;

%rompatch = ( 0x01d8 => 80,
	      0x0211 => 80,
	      0x024e => 80,
	      0x026f => 79,
	      0x02de => 80,
	      0x033c => 80,
	      0x22f2 => 160 );

for ( $i = 0 ; $i < 24; $i++ ) {
    $a = (($i >> 3)*5 << 4) + (($i & 7) << 8);
    $a |= ($a & 0x400) ? 0x7c00 : 0x7400;

    $rompatch{884+2*$i} = ($a & 0xff);
    $rompatch{885+2*$i} = ($a >> 8) & 0xff;
}

# Create patch rom
$patchwidth = 256;
@patchrom = (0xffff) x $patchwidth;
@patched  = (undef) x $patchwidth;
foreach $a ( keys(%rompatch) ) {
    $p = $a % $patchwidth;
    if ( defined($patched[$p]) ) {
	printf STDERR "Address 0x%04x conflicts with 0x%04x\n", $a, $patched{$p};
	exit 1;
    }
    $patched[$p] = $a;
    $patchrom[$p] = (int($a/$patchwidth) << 8) + $rompatch{$a};
}

# Produce MIF file for the patch ROM
open(PMIF, "> basic80.mif");
print PMIF "DEPTH = $patchwidth;\n";
print PMIF "WIDTH = 16;\n";
print PMIF "ADDRESS_RADIX = HEX;\n";
print PMIF "DATA_RADIX    = HEX;\n";
print PMIF "CONTENT\n";
print PMIF "BEGIN\n";
for ( $i = 0 ; $i < $patchwidth ; $i++ ) {
    printf PMIF "%02X : %04X;\n", $i, $patchrom[$i];
}
print PMIF "END;\n";
close(PMIF);
